'' /////////////////////////////////////////////////////////////////////////////
'' Graphics Renderer for Donkey Kong Remake
'' AUTHOR: Steve Waddicor
'  Adapted from: 
'  *****************************
'  *  TV Driver v1.0           *
'  *  (C) 2004 Parallax, Inc.  *
'  *****************************
'' //////////////////////////////////////////////////////////////////////////////

CON
  param_count   = 9

VAR
  long  cog

PUB start(renderer_ptr) : okay

'' Start renderer - starts a cog
'' returns false if no cog available

  okay := (cog := cognew(@entry,renderer_ptr)) > 0

DAT

'******************************
'* Assembly language renderer *
'******************************

                        org
'
'
' Entry
'
entry                   mov     ptr,par                  'load parameters
                        movd    :load,#renderer_params           
                        mov     count,#param_count
:load                   rdlong  renderer_params,ptr
                        add     :load,inc_dest
                        add     ptr,#4
                        djnz    count,#:load               

                        mov     current_line,rend_cog_number  'start rendering from the top

do_line                 movd    :scan_line_store,#cog_scanline_buffer
                        mov     hub_map_ptr,rend_map    'point to first tile (upper-leftmost)

                        mov     tile_row,current_line   'then move down to current line first tile
                        shr     tile_row,#3             '8 lines per tile
                        shl     tile_row,#5             '32 tiles per row
                        add     hub_map_ptr,tile_row

                        mov     tile_line,current_line  'line offset within tile
                        and     tile_line,#$07          '8 lines per tile
                        shl     tile_line,#1            '2 bytes per line
                        mov     tile_table,rend_tiletable  'get a local copy of tiletable address
                        add     tile_table,tile_line    'offset it to read correct tile_line
                        
                        mov     x,rend_ht               'set horizontal tiles
                        shr     x,#1                    '16 pixels at a time from 8 pixel tiles
                        
:do_tile_pair           rdbyte  tile,hub_map_ptr        'read tile
                        shl     tile,#4                 '16 bytes per tile
                        add     tile,tile_table         'convert to pointer
                        rdword  pixels1,tile            '(2 instructions between reads)
                        add     hub_map_ptr,#1          'point to next tile (one instruction between reads)

                        rdbyte  tile,hub_map_ptr        'read tile
                        shl     tile,#4                 '16 bytes per tile
                        add     tile,tile_table         'convert to pointer 
                        rdword  pixels2,tile            '(2 instructions between reads)
                        add     hub_map_ptr,#1          'point to next tile

                        shl     pixels2,#16
                        or      pixels1,pixels2         'final long to write  - combination of 2 tiles
:scan_line_store        mov     cog_scanline_buffer,pixels1
                        add     :scan_line_store,inc_dest 'self-modify dest+1
                                                            
                        djnz    x,#:do_tile_pair  wc      'another tile?

                        ' We've finished rendering the line                        
                        ' We need to wait here for the line we just prepared to be requested by the TV driver
:wait_for_the_request   rdlong  rend_scanline_req_adr,PAR  'requested line is the 1st parameter in the block
                        rdlong  scanline_req,rend_scanline_req_adr
                        cmp     scanline_req,current_line  wc,wz
               if_ne    jmp     #:wait_for_the_request
 
                        ' Our line has been requested so copy it to hub scan line buffer
start_tv_copy           movs    :nextcopy, #cog_scanline_buffer
                        mov     ptr, rend_scanline_buffer
                        mov     count, #16              ' length of visible scanline buffer in longs
:nextcopy               mov     pixels,cog_scanline_buffer
                        add     :nextcopy, #1
                        wrlong  pixels, ptr
                        add     ptr, #4
                        djnz    count, #:nextcopy

                        ' We've handed over the scanline to the TV driver, so do another.    
                        add     current_line,rend_cog_count 'we only do every Nth line
                        cmp     current_line,#224 wc
              if_b      jmp     #do_line
                        mov     current_line,rend_cog_number
                        jmp     #do_line

'
' Initialized data
'
inc_dest                long    1 << 9 << 0

'
' Uninitialized data
'
x                       res     1                       'display
tile                    res     1
pixels1                 res     1
pixels2                 res     1

DAT  ' new stuff

ptr                     res     1
count                   res     1
pixels                  res     1

hub_map_ptr             res     1
current_line            res     1
tile_table              res     1
tile_line               res     1
tile_row                res     1

' Parameter buffer
renderer_params
rend_scanline_req_adr   res     1
rend_map                res     1               
rend_tiletable          res     1
                        res     1               'rend_colors
rend_ht                 res     1               '  
rend_vt                 res     1
rend_scanline_buffer    res     1
rend_cog_number         res     1
rend_cog_count          res     1 
' End of parameter buffer

scanline_req
                        res     1

' Scanline buffer = 256 pixels, 2 bits per pixel = 512 bits = 64 bytes = 16 longs
' However, sprites are going to be 16x16 and I want the facility to be able to slide sprites off
' screen to the left or right without explicit clipping code or corrupting adjacent cog_memory
' So an extra 16 pixels = 32 bits = 1 long either side of the visible scan buffer
                        res     1
cog_scanline_buffer
                        long    52398475
                        long    52398475
                        long    52398475
                        long    0
                        long    52398475
                        long    52398475
                        long    52398475
                        long    52398475
                        res     1
' End of scanline buffer

               