' /////////////////////////////////////////////////////////////////////////////
' Donkey Kong Remake
' AUTHOR: Steve Waddicor
'//////////////////////////////////////////////////////////////////////////////

CON

  _clkmode = xtal1 + pll8x
  _xinfreq = 10_000_000 + 0000
  _stack = 100 'accomodate display memory and stack

  x_tiles = 32
  y_tiles = 28

  tv_paramcount = 17
  rend_paramcount = 9     

VAR
  ' TV Parameter block
  ' Initializing data is in the DAT block at the end of this file.
  ' This is copied in to this variable space with a longmove
  ' so it can be altered easily (without remembering to use @ all the time).
  ' The TV driver then copies these values to it's own cog_memory cache
  ' when it's got a little spare time, several times per frame
  ' so these values can be altered on the fly. 
  long  tv_status     '0/1/2 = off/visible/invisible           read-only
  long  tv_enable     '0/? = off/on                            write-only
  long  tv_pins       '%ppmmm = pins                           write-only
  long  tv_mode       '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen     'pointer to screen (bytes)               write-only
  long  tv_colors     'pointer to colors (longs)               write-only               
  long  tv_hc         'horizontal cells                        write-only
  long  tv_vc         'vertical cells                          write-only
  long  tv_hx         'horizontal cell expansion               write-only
  long  tv_vx         'vertical cell expansion                 write-only
  long  tv_ho         'horizontal offset                       write-only
  long  tv_vo         'vertical offset                         write-only
  long  tv_broadcast  'broadcast frequency (Hz)                write-only
  long  tv_auralcog   'aural fm cog                            write-only
  long  tv_tiletable  'pointer to tile definitions (words)     write-only
  long  tv_scanline_buffer_adr 'a pointer to the fixed scanline buffer                      write-only
  long  tv_scanline_req_adr    'a pointer to the request to put a line in the scan buffer   write-only

  ' Renderer parameter block
  ' Initializing data is in the DAT block at the end of this file.
  ' This is copied in to this variable space with a longmove
  ' so it can be altered easily (without remembering to use @ all the time).
  ' Each renderer then copies these values to it's own cog_memory cache
  ' when it's got a little spare time, several times per frame
  ' so these values can be altered on the fly. 
  long  rend_scanline_req_adr    'a pointer to the request to put a line in the scan buffer   write-only
  long  rend_map                 'pointer to screen (bytes)                                write-only
  long  rend_tiletable           'pointer to tile definitions (words)                      write-only
  long  rend_colors              'pointer to colors (longs)                                write-only               
  long  rend_ht                  'horizontal tiles                                         write-only
  long  rend_vt                  'vertical tiles                                           write-only
  long  rend_scanline_buffer_adr 'hug memory where renderer puts scanline when requested   write-only
  long  rend_cog_number          'each cog gets a different number, starting with zero     write-only
  long  rend_cog_count           'total number of cogs                                     write-only
 
  long  colors[64]

OBJ

  tv    : "sw_dk_tv_drv_013.spin"
  gfx   : "sw_dkong_gfx_001.spin"
  rend  : "sw_dk_gfx_renderer_002.spin"

PUB start | i

  repeat i from 0 to 1
    ' start a renderer
    longmove(@rend_scanline_req_adr, @rend_params, rend_paramcount)
    rend_tiletable := gfx.tile_adr
    rend_map := gfx.map_adr
    rend_scanline_buffer_adr := @hub_scanline_buffer
    rend_scanline_req_adr := @scanline_req
    rend_cog_number := i
    rend_cog_count := 2
    rend.start(@rend_scanline_req_adr)
    repeat 10000

  'start tv
  longmove(@tv_status, @tv_params, tv_paramcount)
  tv_screen := gfx.map_adr
  tv_tiletable := gfx.tile_adr
  tv_colors := @colors
  tv_scanline_buffer_adr := @hub_scanline_buffer
  tv_scanline_req_adr := @scanline_req
  tv.start(@tv_status)

  'init colors
  colors[0] := $48bb4e02

  repeat

DAT

tv_params               long    0                       'status
                        long    1                       'enable
                        long    %011_0000               'pins
                        long    %0000                   'mode
                        long    0                       'screen
                        long    0                       'colors
                        long    x_tiles                 'hc
                        long    y_tiles                 'vc
                        long    10                      'hx
                        long    1                       'vx
                        long    0                       'ho
                        long    0                       'vo
                        long    60_000_000'_xinfreq<<4  'broadcast
                        long    0                       'auralcog
                        long    0                       'tiletable
                        long    0                       'scanline buffer
                        long    scanline_req            'tv_scanline_request

rend_params             long    scanline_req            'rend_scanline_req_adr
                        long    0                       'rend_map
                        long    0                       'rend_tiletable
                        long    0                       'rend_colors
                        long    x_tiles                 'rend_ht
                        long    y_tiles                 'rend_vt
                        long    0                       'rend_hub_scanline_buffer
                        long    0                       'rend_cog_number
                        long    1                       'rend_cog_count

scanline_req
                        long    510
                        
' Scanline buffer = 256 pixels, 2 bits per pixel = 512 bits = 64 bytes = 16 longs
' However, sprites are going to be 16x16 and I want the facility to be able to slide sprites off
' screen to the left or right without explicit clipping code or corrupting adjacent cog_memory
' So an extra 16 pixels = 32 bits = 1 long either side of the visible scan buffer
                        res     1  
hub_scanline_buffer
                        res 16
                        res     1
' End of scanline buffer